﻿/**
 * @class Component
 *
 * @description Eve Manager Component Class
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function Component(){
	//Instance variables (private)
	var self = this;
	
	var init = function(){
        //In test (array con le referenze di Unique ID)
        self.addParamVal("interfacesLinkedUIComponents",new Array());
    };
	
	/** Component Id
	 * @public
	 * @default undefined
	 */
	this.id = undefined;
	
	/** Component Unique Id
     * @private 
     * @type Integer
     * @default undefined
     */
    this.UId = undefined;
    
    /** Component parent Unique Id
     * @private 
     * @type Integer
     * @default undefined
     */
    this.parentUId = undefined;
	
	/** @public @description Component name @default undefined */
	//this.name = undefined;
	
	/** Component type
	 * @public 
	 * @type string
	 * @default undefined
	 */
	this.type = undefined;


    /** Component tag
     * @public
     * @type string
     * @default undefined
     */
    this.tag = undefined;

	/** @public @description Component children nodes */
	//this.m_Components = new Node();											//Node of child components if it is a group
	
	/** Component data (group data contains only group name)
	 * @public
	 * @type object
	 * @default new Object()
	 */
	this.m_Data = new Object();
	
	/** Component description object
	 * @public
	 * @type EveComponentDescriptor
	 * @default new Object()
	 */
	this.m_oDescriptor = new ComponentDescriptor();						      //Component actions descriptor
	
	//Methods
	
	/** Add new parameter and set value - Register action
	 * @param {string} key Key of memory area
	 * @param {string|Number} value of m_Data[Key]
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 */
	this.addParamVal = function(key,value){
		//Check if key already exist
		if(self.m_Data[key] != undefined){ return 1; }
		
		self.m_Data[key] = value;
		return 0;
	};
	
	/** Set parameter value
	 * @param {string} key Key of memory area
	 * @param {string|Number} value of m_Data[Key]
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 */
	this.setParamVal = function(key,value){
		// Check if key already exist []
		if(self.m_Data[key] == undefined){
            this.addParamVal(key,value);
        }
		
		self.m_Data[key] = value;
		return 0;
	};
	
	/** Get parameter value
	 * @param {string} key Key of memory area
	 * @returns {mixed}
	 */
	this.getParamVal = function(key){
	    return self.m_Data[key];
	};
	
	/** Remove parameter from data
     * @param {string} key Key of memory area
     * @returns {Integer} Error description (0 = Ok; otherwise = Error)
     */
	this.removeParam = function(key){
	    delete self.m_Data[key];
	};
	
	/** Set component type
	 * @param str type of component
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 */
	this.setType = function(strType){
		//Check input parameter is a string
		if(typeof strType != "string"){
			console.error("Type of input parameter is not a string");
			return 1;
		}
		
		//Check if type member is already defined (can be defined only once)
		if(self.type != undefined){
			console.error("Type parameter is already defined");
			return 2 ;
		}
		
		self.type = strType;
		return 0;
	};

    /** Set component tag
     * @param str Tags of component
     * @returns {Integer} Error description (0 = Ok; otherwise = Error)
     */
    this.setTag = function(strTag){
        //Check input parameter is a string
        if(typeof strTag != "string"){
            console.error("Type of input parameter is not a string");
            return 1;
        }

        self.tag = strTag;
        return 0;
    };

    /** Check if a tag is included
     * @param str Tag to check
     * @returns 0 if found , 1 if not found
     */
    this.includeTag = function(strTag){
        //Check input parameter is a string
        var tt=self.getParamVal("tags");// console.log(self);
        if (tt != undefined )
        {
            if (tt.indexOf(strTag)>-1)
                return true;
        }
        return false;

    };


	/** Get component type
	 * @returns {string} Type of component
	 */
	this.getType = function(){
		return self.type;
	};

    /** Get component tags
     * @returns {string} tags of component
     */
    this.getTag = function(){
        return self.tag;
    };

	/** Get descriptor object
	 * @returns {EveComponentDescriptor} Descriptor object
	 */
	this.getDescriptor = function(){
	    return self.m_oDescriptor;
	};
	
	/** JSON serialization
	 *
	 */
	this.encodeJSON = function(){
        
        //Set instance variables
        var oSerialized = {
            UId: self.UId,
            parentUId: self.parentUId,
            type: self.type,
            descriptor: self.m_oDescriptor.encodeJSON()
        };
        //console.log(this);
        //console.log(self.m_Data);

        //Set data parameters
        for(var k in self.m_Data){
            if(k == "interfacesLinkedUIComponents"){
                continue;
            }
            else if(k == "gateway"){
                //var oGatewayNode = self.m_Data[k];

                // Commented line , check with checkMatteo
             //   console.log(self);
                try
                {
                    oSerialized[k] = self.m_Data[k].getData().UId;
                }
                catch (er)
                {
                    console.log("Gateway missing on component uid: "+self.m_Data.UId);
                }

            }
            else if(k == "parentUId"){
                // Dario per evitare/sistemare errori di posizionamento dei componenti nella lista
            }
            else if(self.m_Data[k] instanceof Array){
                //var oGatewayNode = self.m_Data[k];
                oSerialized[k] = self.m_Data[k].slice();
            }
            else{
                oSerialized[k] = self.m_Data[k];
            }

            //console.error(self.m_Data[k] instanceof Array);
        }
        
        return oSerialized;
    };
	
	
	/** Overwrite toString() method
     * @returns {string} String that describe Component object
     */
    this.toString = function(){
        //--console.log("component toString");
        return "[Component] - Unique Id: "+ self.UId + "; name: "+ self.m_Data["name"] + "; type: "+ self.type;
    };

    /** Initial operations */
    init();
};