﻿/** Javascript Clivet ElfoControl2 Importer
 *
 * 5/3/2014 by Andreosky Padovanosky
 *
 */
"use strict";

var VERSION = "v.1.0.2";
var NAME = "Clivet Elfo Control 2 " + VERSION;
var $ = jQuery;

try {
    console.log("%cStart " + NAME, "color:DarkSlateBlue");
    var xmlDoc = $.parseXML(QT.readAnyFile());
    // Check main root node
    var oRoot = xmlDoc.childNodes[0];
    if (oRoot.nodeName != "installer.ConfigurazioneInstallatore") {
        throw "Invalid Xml File, main node installer.ConfigurazioneInstallatore not found.";
    }

    // Global var
    var gtwName = "ElfoControl2";
    var unitID = 190;
    var oGateway = getGateway(gtwName);
    var oMainGroup = getMainGroup(gtwName);

    blr.executionStack.show(NAME);

    // Create heart bit
    blr.executionStack.push(
        function (in_args,out_args) {
            out_args = createHeartbit(
                in_args.gateway,
                in_args.id,
                in_args.group
            );
        },
        {
            gateway:oGateway,
            id:unitID,
            group:oMainGroup
        }
    );

    // Create system
    blr.executionStack.push(
        function (in_args,out_args) {
            out_args = createSystem(
                in_args.gateway,
                in_args.id,
                in_args.group
            );
        },
        {
            gateway:oGateway,
            id:unitID,
            group:oMainGroup
        }
    );

    // Create System ACS
    //createSystemACS(oGateway, unitID, oMainGroup);

    // Create Elfo Fresh

    // Create Zone parameter
    var aZones = oRoot.getElementsByTagName("zone")[0].getElementsByTagName("installer.elementi.InstallatoreZona");
    for (var z = 0; z < aZones.length; ++z) {
        var oZone = aZones[z];
        var idZona = oZone.getElementsByTagName("idZona")[0].childNodes[0];
        var nameZona = oZone.getElementsByTagName("nome")[0].childNodes[0];
        blr.executionStack.push(
            function (in_args,out_args) {
                out_args = createZone(
                    in_args.gateway,
                    in_args.id,
                    in_args.group,
                    in_args.zoneID,
                    in_args.zoneName
                );
            },
            {
                gateway:oGateway,
                id:unitID,
                group:oMainGroup,
                zoneID:idZona.nodeValue,
                zoneName:nameZona.nodeValue
            }
        );
    }

    blr.executionStack.execute(10,true);

    console.log("%cEnd " + NAME, "color:DarkSlateBlue");
} catch (err) {
    console.error(NAME + " error\n" + err);
    alert(NAME + " error\n" + err);
}

function createZone(gateway, unit, group, id, name) {
    var oItem = null;
    var oGroup = application.addComponent({
        type: "group",
        name: id + "." + name
    }, group);

    var startRegister = 150;
    var countRegisters = 48;
    var range = 4;
    var offsetBits = (id-1) * range * 16;

	var startRegisterWriteRead = 10;
    var countRegistersWriteRead = 42;
    var rangeWrite = 7;
    var offsetRegisterWrite = ((id - 1) * rangeWrite);
    var offsetBitsWrite = (id - 1) * rangeWrite * 16;

    if(id > 6){
    	startRegisterWriteRead += countRegistersWriteRead;
    	offsetRegisterWrite = ((id - 1) * rangeWrite);
    	offsetBitsWrite = (id - 7) * rangeWrite * 16;
    }

	///////////////////////////////////////////
    // States
    ///////////////////////////////////////////
    // Temperatura media Zona
    oItem = application.addComponent({
        type: "info",
        gateway : gateway,
        name : _D("Average Temperature"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start" : startRegister,
        "modbustcp/read-register-count" : countRegisters,
        "modbustcp/read-bit-start": offsetBits + 0 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 5,// Int16
        "modbustcp/read-transform-multiplier" : 0.1
    }, oGroup);
    // Umidità
    oItem = application.addComponent({
        type: "info",
        gateway: gateway,
        name: _D("Average Humidity"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegister,
        "modbustcp/read-register-count": countRegisters,
        "modbustcp/read-bit-start": offsetBits + 1 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-multiplier": 1
    }, oGroup);
    // Profilo
    oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Profile State (Read Only)"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegister,
        "modbustcp/read-register-count": countRegisters,
        "modbustcp/read-bit-start": offsetBits + 2 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 3,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 14 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 3,
        "modbustcp/write-transform-multiplier": 1,
        "values": [{
            "value": 1,
            "description": _D("Off")
        }, {
            "value": 2,
            "description": _D("Economy")
        }, {
            "value": 3,
            "description": _D("Comfort")
        }],
        "icons": [
            blr.gfxResource.getIconByTypeName(1, "off"),
            blr.gfxResource.getIconByTypeName(2, "economy"),
            blr.gfxResource.getIconByTypeName(3, "comfort")
        ]
    }, oGroup);

    // Set Point
    oItem = application.addComponent({
        type: "info",
        gateway: gateway,
        name: _D("Setpoint"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegister,
        "modbustcp/read-register-count": countRegisters,
        "modbustcp/read-bit-start": offsetBits + 3 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-multiplier": 0.1
    }, oGroup);

    ///////////////////////////////////////////
    // Commands
    ///////////////////////////////////////////
    // Winter setpoint
    oItem = application.addComponent({
        type: "thermostat",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Winter setpoint"),
        step:0.1,
        min:8,
        max:35,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBitsWrite + 0 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 80,
        "modbustcp/read-transform-max": 350,
        "modbustcp/read-transform-multiplier": 0.1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 10 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 80,
        "modbustcp/write-transform-max": 350,
        "modbustcp/write-transform-multiplier": 0.1
    }, oGroup);
    oItem = application.addComponent({
        type: "timed-thermostat",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Winter setpoint timed"),
        step:0.1,
        min:8,
        max:35,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBitsWrite + 0 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 80,
        "modbustcp/read-transform-max": 350,
        "modbustcp/read-transform-multiplier": 0.1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 10 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 80,
        "modbustcp/write-transform-max": 350,
        "modbustcp/write-transform-multiplier": 0.1
    }, oGroup);

    // Summer setpoint
    oItem = application.addComponent({
        type: "thermostat",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Summer setpoint"),
        step:0.1,
        min:8,
        max:35,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBitsWrite + 1 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 80,
        "modbustcp/read-transform-max": 350,
        "modbustcp/read-transform-multiplier": 0.1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 11 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 80,
        "modbustcp/write-transform-max": 350,
        "modbustcp/write-transform-multiplier": 0.1
    }, oGroup);
    oItem = application.addComponent({
        type: "timed-thermostat",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Summer setpoint timed"),
        step:0.1,
        min:8,
        max:35,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBitsWrite + 1 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 80,
        "modbustcp/read-transform-max": 350,
        "modbustcp/read-transform-multiplier": 0.1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 11 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 80,
        "modbustcp/write-transform-max": 350,
        "modbustcp/write-transform-multiplier": 0.1
    }, oGroup);

    // Winter difference setpoint
    oItem = application.addComponent({
        type: "thermostat",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Winter setpoint difference"),
        step:0.1,
        min:0.1,
        max:15,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBitsWrite + 2 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 150,
        "modbustcp/read-transform-multiplier": 0.1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 12 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 150,
        "modbustcp/write-transform-multiplier": 0.1
    }, oGroup);

    // Summer difference setpoint
    oItem = application.addComponent({
        type: "thermostat",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Summer setpoint difference"),
        step:0.1,
        min:0.1,
        max:15,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBitsWrite + 3 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 150,
        "modbustcp/read-transform-multiplier": 0.1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 13 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 150,
        "modbustcp/write-transform-multiplier": 0.1
    }, oGroup);

    // Profile
    oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Profile"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBits + 4 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 3,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 14 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 3,
        "modbustcp/write-transform-multiplier": 1,
        "values": [{
            "value": 1,
            "description": _D("Off")
        }, {
            "value": 2,
            "description": _D("Economy")
        }, {
            "value": 3,
            "description": _D("Comfort")
        }],
        "icons": [
            blr.gfxResource.getIconByTypeName(1, "off"),
            blr.gfxResource.getIconByTypeName(2, "economy"),
            blr.gfxResource.getIconByTypeName(3, "comfort")
        ]
    }, oGroup);

    // Winter Humidity
    oItem = application.addComponent({
        type: "tuner",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Winter humidity"),
        step:1,
        min:10,
        max:90,
        digits:0,
        suffix:"%",
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBitsWrite + 5 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 10,
        "modbustcp/read-transform-max": 90,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 15 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 10,
        "modbustcp/write-transform-max": 90,
        "modbustcp/write-transform-multiplier": 1
    }, oGroup);

    // Summer Humidity
    oItem = application.addComponent({
        type: "tuner",
        gateway: gateway,
        savelastvalue: true,
        name: _D("Summer humidity"),
        step:1,
        min:10,
        max:90,
        digits:0,
        suffix:"%",
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": startRegisterWriteRead,
        "modbustcp/read-register-count": countRegistersWriteRead,
        "modbustcp/read-bit-start": offsetBitsWrite + 6 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 10,
        "modbustcp/read-transform-max": 90,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 16 + offsetRegisterWrite,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 10,
        "modbustcp/write-transform-max": 90,
        "modbustcp/write-transform-multiplier": 1
    }, oGroup);

    return oGroup;
}

function createElfoFresh(gateway, unit, group) {
    var oGroup = application.addComponent({
        type: "group",
        name: _D("Elfo Fresh")
    }, group);

    return oGroup;
}

function createSystemACS(gateway, unit, group) {
    var oGroup = application.addComponent({
        type: "group",
        name: _D("System ACS")
    }, group);
    return oGroup;
}

function createSystem(gateway, unit, group) {
    var oItem = null;
    var oGroup = application.addComponent({
        type: "group",
        name: _D("System")
    }, group);

    // Stato impianto
    oItem = application.addComponent({
        type: "switch",
        gateway: gateway,
        name: _D("State"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 140,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 0 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 2,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 1,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 2,
        "modbustcp/write-transform-multiplier": 1,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "off"),
            blr.gfxResource.getIconByTypeName(1, "on")
        ]
    }, oGroup);

    // Modo
    oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        name: _D("Mode"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 140,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 1 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 2,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 2,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 2,
        "modbustcp/write-transform-multiplier": 1,
        "values": [{
            "value": 1,
            "description": _D("Cold")
        }, {
            "value": 2,
            "description": _D("Hot")
        }],
        "icons": [
            blr.gfxResource.getIconByTypeName(1, "cold",0,0,200),
            blr.gfxResource.getIconByTypeName(2, "hot",200)
        ]
    }, oGroup);

	// Stato PDC
	oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        name: _D("PDC state (Read Only)"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 140,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 2 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 0,
        "modbustcp/read-transform-max": 2,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 3,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 0,
        "modbustcp/write-transform-max": 2,
        "modbustcp/write-transform-multiplier": 1,
        "values": [{
            "value": 0,
            "description": _D("Off")
        }, {
            "value": 1,
            "description": _D("Work for system")
        }, {
            "value": 2,
            "description": _D("Work for ACS")
        }],
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "off",128,128,128),
            blr.gfxResource.getIconByTypeName(1, "work"),
            blr.gfxResource.getIconByTypeName(2, "water",200,0,0)
        ]
    }, oGroup);
	
	// Pompa calore 
	oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        name: _D("PDC mode"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 1,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 2 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 3,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 3,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 3,
        "modbustcp/write-transform-multiplier": 1,
        "values": [{
            "value": 1,
            "description": _D("Disable")
        }, {
            "value": 2,
            "description": _D("ACS")
        }, {
            "value": 3,
            "description": _D("Automatic")
        }],
        "icons": [
            blr.gfxResource.getIconByTypeName(1, "disable",128,128,128),
            blr.gfxResource.getIconByTypeName(2, "water",200,0,0),
            blr.gfxResource.getIconByTypeName(3, "automatic")
        ]
    }, oGroup);

	// ACS mode 
	oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        name: _D("ACS mode"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 1,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 3 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 4,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 4,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 4,
        "modbustcp/write-transform-multiplier": 1,
        "values": [{
            "value": 1,
            "description": _D("Disable")
        }, {
            "value": 2,
            "description": _D("Sun")
        }, {
            "value": 3,
            "description": _D("Aux")
        }, {
            "value": 4,
            "description": _D("Automatic")
        }],
        "icons": [
            blr.gfxResource.getIconByTypeName(1, "disable", 128, 128, 128),
            blr.gfxResource.getIconByTypeName(2, "sun", 200, 200, 0),
            blr.gfxResource.getIconByTypeName(3, "aux"),
            blr.gfxResource.getIconByTypeName(4, "automatic"),
        ]
    }, oGroup);

	// Out Home
	oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        name: _D("Out/In Home"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 1,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 4 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 2,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 5,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 2,
        "modbustcp/write-transform-multiplier": 1,
        "values": [{
            "value": 1,
            "description": _D("Out")
        }, {
            "value": 2,
            "description": _D("In")
        }],
        "icons": [
            blr.gfxResource.getIconByTypeName(1, "home_out"),
            blr.gfxResource.getIconByTypeName(2, "home_in"),
        ]
    }, oGroup);

    /* Change with Scenario
	// Reset Alarms
	oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        name: _D("Alarms reset"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 1,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 5 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 1,
        "modbustcp/read-transform-max": 3,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 6,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 1,
        "modbustcp/write-transform-max": 3,
        "modbustcp/write-transform-multiplier": 1,
        "values": [{
            "value": 1,
            "description": _D("PDC")
        }, {
            "value": 2,
            "description": _D("Fresh")
        }, {
            "value": 3,
            "description": _D("Fresh 2")
        }]
    }, oGroup);
    */
	// Winter PDC setpoint
    oItem = application.addComponent({
        type: "thermostat",
        gateway: gateway,
        name: _D("Winter PDC"),
        step:0.1,
        min:5,
        max:30,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 1,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 6 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 50,
        "modbustcp/read-transform-max": 300,
        "modbustcp/read-transform-multiplier": 0.1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 7,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 50,
        "modbustcp/write-transform-max": 300,
        "modbustcp/write-transform-multiplier": 0.1
    }, oGroup);

	// Summer PDC setpoint
    oItem = application.addComponent({
        type: "thermostat",
        gateway: gateway,
        name: _D("Summer PDC"),
        step:0.1,
        min:20,
        max:60,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 1,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 7 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4, // word
        "modbustcp/read-transform-min": 200,
        "modbustcp/read-transform-max": 600,
        "modbustcp/read-transform-multiplier": 0.1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 8,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 4, // word
        "modbustcp/write-transform-min": 200,
        "modbustcp/write-transform-max": 600,
        "modbustcp/write-transform-multiplier": 0.1
    }, oGroup);

	// Temperatura aria esterna
    oItem = application.addComponent({
        type: "info",
        gateway: gateway,
        name: _D("Out Temperature"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 140,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 3 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 5,// Int16
        "modbustcp/read-transform-multiplier": 0.1,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "temperature")
        ]
    }, oGroup);

    // PDC Alarm
    oItem = application.addComponent({
        type: "info-text",
        gateway: gateway,
        name: _D("PDC Alarm"),
        "text-0":"OK",
        "text-1": "Alarm",
        "color-0":"#00C800",
        "color-1": "#C80000",
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 140,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 4 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 1,// Bit
        "modbustcp/read-transform-multiplier": 1,
        "icons": [
            blr.gfxResource.getIconByTypeName(1, "circle"),
            blr.gfxResource.getIconByTypeName(2, "triangle")
        ]
    }, oGroup);

    // PDC Deliver Temperature
    oItem = application.addComponent({
        type: "info",
        gateway: gateway,
        name: _D("PDC Deliver Temperature"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 140,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 5 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 5,// Int16
        "modbustcp/read-transform-multiplier": 0.1,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "temperature")
        ]
    }, oGroup);

    // PDC Power
    oItem = application.addComponent({
        type: "info",
        gateway: gateway,
        name: _D("PDC Power"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 140,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 6 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 4,// word
        "modbustcp/read-transform-multiplier": 0.1,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "power")
        ]
    }, oGroup);

	// PDC Set Point
    oItem = application.addComponent({
        type: "info",
        gateway: gateway,
        name: _D("PDC Setpoint"),
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 140,
        "modbustcp/read-register-count": 8,
        "modbustcp/read-bit-start": 7 * 16,
        "modbustcp/read-bit-count": 16,
        "modbustcp/read-transform-type": 5,// Int16
        "modbustcp/read-transform-multiplier": 0.1,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "temperature")
        ]
    }, oGroup);

    return oGroup;
}

function createHeartbit(gateway, unit, group) {
    var oGroup = application.addComponent({
        type: "group",
        name: _D("Heartbit")
    }, group);

    var oHeartbit = application.addComponent({
        type: "switch",
        gateway: gateway,
        name: _D("Heartbit"),
        savelastvalue: true,
        cache:true,
        "modbustcp/read-unit-id": unit,
        "modbustcp/read-function-id": 3, // (3) Read Holding Registers
        "modbustcp/read-register-start": 0,
        "modbustcp/read-register-count": 1,
        "modbustcp/read-bit-start": -1,
        "modbustcp/read-bit-count": 0,
        "modbustcp/read-transform-type": 1, // bit
        "modbustcp/read-transform-min": 0,
        "modbustcp/read-transform-max": 1,
        "modbustcp/read-transform-multiplier": 1,
        "modbustcp/write-unit-id": unit,
        "modbustcp/write-function-id": 6, // (6) Write Single Register
        "modbustcp/write-register-start": 0,
        "modbustcp/write-register-count": 1,
        "modbustcp/write-bit-start": -1,
        "modbustcp/write-bit-count": 0,
        "modbustcp/write-transform-type": 1, // bit
        "modbustcp/write-transform-min": 0,
        "modbustcp/write-transform-max": 1,
        "modbustcp/write-transform-multiplier": 1
    }, oGroup);

    var oStartHeartbit = application.addComponent({
        type: "switch",
        name: _D("Start Heartbit"),
        savelastvalue: true,
        cache: true
    }, oGroup);

    var oLoopHeartbit = application.addComponent({
        type: "scenario",
        name: _D("Loop Heartbit"),
        cache: true
    }, oGroup);

    var oCompositor = oLoopHeartbit.data.getParamVal("ghost");
    oCompositor.attached = {
        UId: oStartHeartbit.getData().UId,
        loop: true,
        reverse:false
    };
    
    oCompositor.actions = [
        {
            UId: oHeartbit.getData().UId,
            action: "1",
            value: "1"
        },
        {
            UId: "eve-wait",
            action: "1000",
            value: "10"
        }
    ];
    return oGroup;
}

function getMainGroup(name) {
    var oGroup = application.addComponent({
        type: "group",
        name: name
    });
    return oGroup;
}

function getGateway(name) {
    var oGateway = undefined;
    var gtwIp = "10.0.0.141";
    var gtwPort = 23;

    // Create Gateway
    var aGateways = application.getGatewaysByProtocol("modbustcp");
    for (var g = 0; g < aGateways.length; ++g) {
        if (aGateways[g].data.getName() == name) {
            oGateway = aGateways[g];
            break;
        }
    }

    if (oGateway == undefined) {
        oGateway = application.addGateway({
            name: name,
            protocol: "modbustcp"
        });

        // Default setting
        oGateway.data.setParamVal("modbustcp/address", gtwIp);
        oGateway.data.setParamVal("modbustcp/port", gtwPort);
    }

    return oGateway;
}